import React, { useEffect, forwardRef, useState, useImperativeHandle, useContext, createContext, useLayoutEffect, useMemo, useCallback, useRef } from 'react';
import * as THREE from 'three';
import { WebGLRenderer, sRGBEncoding, PCFSoftShadowMap, Scene, Object3D, Material, BufferGeometry, Fog, WebGLRenderTarget, HalfFloatType, LinearEncoding, Vector2, Raycaster, TextureLoader, Group, InstancedMesh, DynamicDrawUsage } from 'three';
import { AdaptiveToneMappingPass } from 'three/examples/jsm/postprocessing/AdaptiveToneMappingPass.js';
import { useConst } from '@hmans/react-toolbox';
import { EffectComposer } from 'three/examples/jsm/postprocessing/EffectComposer';
import { RenderPass as RenderPass$1 } from 'three/examples/jsm/postprocessing/RenderPass';
import { UnrealBloomPass as UnrealBloomPass$1 } from 'three/examples/jsm/postprocessing/UnrealBloomPass';
import { ShaderPass } from 'three/examples/jsm/postprocessing/ShaderPass';
import { VignetteShader } from 'three/examples/jsm/shaders/VignetteShader';
import { Tag } from 'miniplex';
import { createECS } from 'miniplex-react';
import { Text as Text$1 } from 'troika-three-text';

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = _objectWithoutPropertiesLoose(source, excluded);
  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

var useWindowResizeHandler = function useWindowResizeHandler(fun) {
  var deps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  useEffect(function () {
    /* Invoke once */
    fun();
    /* Invoke every time the window is resized */

    window.addEventListener("resize", fun, false);
    return function () {
      return void window.removeEventListener("resize", fun);
    };
  }, [fun].concat(_toConsumableArray(deps)));
};

var RendererContext = /*#__PURE__*/createContext(null);
var Renderer = /*#__PURE__*/forwardRef(function (_ref, ref) {
  var children = _ref.children;

  var _useState = useState(),
      _useState2 = _slicedToArray(_useState, 2),
      canvas = _useState2[0],
      setCanvas = _useState2[1];

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      renderer = _useState4[0],
      setRenderer = _useState4[1];

  useEffect(function () {
    if (!canvas) return;
    setRenderer(function () {
      var renderer = new WebGLRenderer({
        canvas: canvas,
        powerPreference: "high-performance",
        antialias: false,
        alpha: false,
        stencil: false,
        logarithmicDepthBuffer: true
      });
      renderer.autoClear = false;
      renderer.setPixelRatio(1);
      renderer.outputEncoding = sRGBEncoding;
      renderer.shadowMap.enabled = true;
      renderer.shadowMap.type = PCFSoftShadowMap;
      renderer.setSize(canvas.clientWidth, canvas.clientHeight);
      return renderer;
    });
    return function () {
      return void setRenderer(undefined);
    };
  }, [canvas]);
  useImperativeHandle(ref, function () {
    return renderer;
  }, [renderer]);
  useWindowResizeHandler(function () {
    if (!renderer) return;
    var width = window.innerWidth;
    var height = window.innerHeight;
    renderer.setSize(width, height);
  }, [renderer]);
  return /*#__PURE__*/React.createElement("canvas", {
    ref: setCanvas
  }, renderer && /*#__PURE__*/React.createElement(RendererContext.Provider, {
    value: renderer
  }, typeof children === "function" ? children() : children));
});
var useRenderer = function useRenderer() {
  return useContext(RendererContext);
};

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (_isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _toArray(arr) {
  return _arrayWithHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableRest();
}

/**
 * Convenience method for setting (potentially nested) properties on an object.
 */
var applyProps = function applyProps(object, props) {
  for (var key in props) {
    var _object$key, _object$key2, _object$key3;

    var value = props[key];
    /* If the key contains a hyphen, we're setting a sub property. */

    if (key.indexOf("-") > -1) {
      var _key$split = key.split("-"),
          _key$split2 = _toArray(_key$split),
          property = _key$split2[0],
          rest = _key$split2.slice(1);

      applyProps(object[property], _defineProperty({}, rest.join("-"), value));
      continue;
    }
    /* Use setScalar if available */


    if ((_object$key = object[key]) !== null && _object$key !== void 0 && _object$key.setScalar && typeof value === "number") {
      object[key].setScalar(value);
      continue;
    }
    /* Use copy if available */


    if ((_object$key2 = object[key]) !== null && _object$key2 !== void 0 && _object$key2.copy && object[key].constructor === value.constructor) {
      object[key].copy(value);
      continue;
    }
    /* Use set if available */


    if ((_object$key3 = object[key]) !== null && _object$key3 !== void 0 && _object$key3.set) {
      var _object$key4;

      Array.isArray(value) ? (_object$key4 = object[key]).set.apply(_object$key4, _toConsumableArray(value)) : object[key].set(value);
      continue;
    }
    /* Otherwise, just directly assign the property. */


    if (key in object) {
      object[key] = value;
    }
  }
};

var ParentContext = /*#__PURE__*/createContext(null);

/**
 * Fetches the scene object that whatever we're rendering here will be
 * parented to.
 *
 * @returns The current parent.
 */

var useParent = function useParent() {
  return useContext(ParentContext);
};

/**
 * Manages the lifecycle of a THREE.* object instance, making sure it gets disposed once
 * the component using it is unmounted.
 */
var useManagedThreeObject = function useManagedThreeObject(fn) {
  var instance = useConst(fn);
  /* Automatically dispose the object if we can */

  useEffect(function () {
    if (!instance || instance instanceof Scene) return;
    return function () {
      if (instance.dispose) {
        instance.dispose();
      }
    };
  }, [instance]);
  return instance;
};

var _excluded$3 = ["object", "children", "attach", "args"];
var makeComponent = function makeComponent(constructor, displayName) {
  /* Create a component that wraps the requested constructible instance */
  var Component = /*#__PURE__*/forwardRef(function (_ref, ref) {
    var propObject = _ref.object,
        children = _ref.children,
        attach = _ref.attach,
        args = _ref.args,
        props = _objectWithoutProperties(_ref, _excluded$3);

    /* Get the current parent. */
    var parent = useParent();
    /* Create the instance of our THREE object. */

    var instance = useManagedThreeObject(function () {
      return propObject || _construct(constructor, _toConsumableArray(Array.isArray(args) ? args : Array(args)));
    });
    /* Apply forwarded ref */

    useImperativeHandle(ref, function () {
      return instance;
    });
    /* Apply props */

    if (props) applyProps(instance, props);
    /* Mount scene object to parent */

    useLayoutEffect(function () {
      if (!parent || !(instance instanceof Object3D)) return;
      parent.add(instance);
      return function () {
        return void parent.remove(instance);
      };
    }, [parent, instance]);
    /* Attach to parent attributes */

    useLayoutEffect(function () {
      if (!parent || !instance) return;
      /* For specific types, set a default attach property */

      if (!attach) {
        if (instance instanceof Material) attach = "material";else if (instance instanceof BufferGeometry) attach = "geometry";else if (instance instanceof Fog) attach = "fog";
      }
      /* If the instance has an "attach" property, attach it to the parent */


      if (attach) {
        if (attach in parent) {
          parent[attach] = instance;
          return function () {
            return void (parent[attach] = undefined);
          };
        } else {
          console.error("Property \"".concat(attach, "\" does not exist on parent \"").concat(parent.constructor.name, "\""));
        }
      }
    }, [instance, attach]);
    return /*#__PURE__*/React.createElement(ParentContext.Provider, {
      value: instance
    }, typeof children === "function" ? children() : children);
  });
  Component.displayName = displayName;
  return Component;
};

var Primitive = makeComponent(undefined, "Primitive");

var cache = {};
/**
 * The Trinity Reactor. For every class exposed by THREE, this object contains a
 * Trinity component that wraps the class (see `makeComponent`.)
 */

function makeReactor(source) {
  return new Proxy({}, {
    get: function get(_, name) {
      /* Create and memoize a wrapper component for the specified property. */
      if (!cache[name]) {
        /* Try and find a constructor within the THREE namespace. */
        var _constructor = source[name];
        /* If nothing could be found, bail. */

        if (!_constructor) return undefined;
        /* Otherwise, create and memoize a component for that constructor. */

        cache[name] = makeComponent(_constructor, name);
      }

      return cache[name];
    }
  });
}

var T$2 = makeReactor(THREE);

/* Named Exports */
var T$1 = T$2;

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}

var useAnimationFrame = function useAnimationFrame(fn) {
  return useLayoutEffect(function () {
    var looping = true;

    var tick = function tick(time) {
      if (looping) requestAnimationFrame(tick);
      fn(time);
    };

    requestAnimationFrame(tick);
    return function () {
      looping = false;
    };
  }, [fn]);
};

var TickerContext = /*#__PURE__*/createContext(null);

var TickerImpl = /*#__PURE__*/function () {
  function TickerImpl() {
    _classCallCheck(this, TickerImpl);

    _defineProperty(this, "timeScale", 1);

    _defineProperty(this, "fixedStep", 1 / 50);

    _defineProperty(this, "maxDelta", 1);

    _defineProperty(this, "lastTime", performance.now());

    _defineProperty(this, "callbacks", new Map());

    _defineProperty(this, "acc", 0);
  }

  _createClass(TickerImpl, [{
    key: "on",
    value: function on(stage, callback) {
      if (!this.callbacks.has(stage)) this.callbacks.set(stage, []);
      this.callbacks.get(stage).push(callback);
    }
  }, {
    key: "off",
    value: function off(stage, callback) {
      var callbacks = this.callbacks.get(stage);
      var pos = callbacks.indexOf(callback);
      callbacks.splice(pos, 1);
    }
  }, {
    key: "tick",
    value: function tick(time) {
      /* Calculate frame delta */
      var frameDelta = (time - this.lastTime) / 1000;
      this.lastTime = time;
      /* Clamp the deltatime to prevent situations where thousands of frames are executed after
      the user returns from another tab. */

      var dt = Math.max(0, this.maxDelta ? Math.min(frameDelta, this.maxDelta) : frameDelta);
      var dtScaled = dt * this.timeScale;
      /* Run early callbacks. */

      this.execute("early", dtScaled);
      this.execute("physics", dtScaled);
      /* Run fixed-steps callbacks, based on our internal accumulator. */

      this.acc += dtScaled;

      while (this.acc >= this.fixedStep) {
        this.acc -= this.fixedStep;
        this.execute("fixed", this.fixedStep);
        this.execute("lateFixed", this.fixedStep);
      }
      /* Continue with normal updates */


      this.execute("update", dtScaled);
      this.execute("lateUpdate", dtScaled);
      /* And finally, execute render callbacks */

      this.execute("render", dtScaled);
    }
  }, {
    key: "execute",
    value: function execute(stage, dt) {
      var callbacks = this.callbacks.get(stage);

      if (callbacks) {
        var _iterator = _createForOfIteratorHelper(callbacks),
            _step;

        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var callback = _step.value;
            callback(dt);
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      }
    }
  }]);

  return TickerImpl;
}();

var Ticker = function Ticker(_ref) {
  var children = _ref.children,
      _ref$timeScale = _ref.timeScale,
      timeScale = _ref$timeScale === void 0 ? 1 : _ref$timeScale;

  var _useState = useState(function () {
    return new TickerImpl();
  }),
      _useState2 = _slicedToArray(_useState, 1),
      ticker = _useState2[0];

  useLayoutEffect(function () {
    ticker.timeScale = timeScale;
    return function () {
      return void (ticker.timeScale = 1);
    };
  }, [ticker, timeScale]);
  useAnimationFrame(ticker.tick.bind(ticker));
  return /*#__PURE__*/React.createElement(TickerContext.Provider, {
    value: ticker
  }, children);
};

var useTicker = function useTicker(stage, callback) {
  var ticker = useContext(TickerContext);
  useLayoutEffect(function () {
    ticker.on(stage, callback);
    return function () {
      return ticker.off(stage, callback);
    };
  });
};

var Update = function Update(_ref) {
  var _ref$stage = _ref.stage,
      stage = _ref$stage === void 0 ? "update" : _ref$stage,
      fun = _ref.children;
  var parent = useParent();
  var renderer = useRenderer();
  useTicker(stage, function (dt) {
    return fun(dt, {
      parent: parent,
      renderer: renderer
    });
  });
  return null;
};

var ComposerContext = /*#__PURE__*/createContext(null);
var Composer = function Composer(_ref) {
  var children = _ref.children;
  var renderer = useRenderer();
  var composer = useMemo(function () {
    return new EffectComposer(renderer, new WebGLRenderTarget(window.innerWidth, window.innerHeight, {
      type: HalfFloatType,
      encoding: LinearEncoding
    }));
  }, [renderer]);
  return /*#__PURE__*/React.createElement(ComposerContext.Provider, {
    value: composer
  }, /*#__PURE__*/React.createElement(Update, {
    stage: "render"
  }, function () {
    return composer.render();
  }), children);
};
var useComposer = function useComposer() {
  return useContext(ComposerContext);
};

function useEventListener(target, event, fn) {
  var deps = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  useEffect(function () {
    target.addEventListener(event, fn);
    return function () {
      return target.removeEventListener(event, fn);
    };
  }, [event, target, fn].concat(_toConsumableArray(deps)));
}

var On = function On(_ref) {
  var _ref$target = _ref.target,
      target = _ref$target === void 0 ? window : _ref$target,
      event = _ref.event,
      children = _ref.children;
  useEventListener(target, event, children);
  return null;
};

var EventHandling = function EventHandling(_ref) {
  var camera = _ref.camera,
      scene = _ref.scene;
  var renderer = useRenderer();
  var pointer = useMemo(function () {
    return new Vector2();
  }, []);
  var raycaster = useMemo(function () {
    return new Raycaster();
  }, []);
  var intersects = new Array();
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(On, {
    target: renderer.domElement,
    event: "pointermove"
  }, useCallback(function (e) {
    /* Get normalized pointer position */
    pointer.x = e.clientX / window.innerWidth * 2 - 1;
    pointer.y = -(e.clientY / window.innerHeight) * 2 + 1;
    /* Prepare raycaster */

    raycaster.setFromCamera(pointer, camera);
    /* Find intersects */

    intersects.length = 0;
    raycaster.intersectObject(scene, true, intersects);
  }, [renderer, raycaster, camera, scene])), /*#__PURE__*/React.createElement(On, {
    target: renderer.domElement,
    event: "pointerdown"
  }, useCallback(function (e) {
    console.log(intersects[0]);
  }, [renderer, raycaster, camera, scene])));
};

var OnWindowResize = function OnWindowResize(_ref) {
  var children = _ref.children;

  /* Invoke the function at least once */
  useEffect(function () {
    return children();
  }, []);
  /* Bind it to the window's resize event */

  return /*#__PURE__*/React.createElement(On, {
    event: "resize",
    target: window,
    children: children
  });
};

var EffectPass = function EffectPass(props) {
  var composer = useComposer();
  var pass = useMemo(function () {
    return _construct(props.pass, _toConsumableArray(props.args));
  }, props.args);
  useEffect(function () {
    composer.addPass(pass);
    return function () {
      return composer.removePass(pass);
    };
  }, [composer, pass]);
  return null;
};

var LensDirtShader = {
  uniforms: {
    tDiffuse: {
      value: null
    },
    tDirt: {
      value: null
    },
    strength: {
      value: 1.0
    },
    minLuminance: {
      value: 0.1
    },
    maxLuminance: {
      value: 0.5
    }
  },
  vertexShader: "\n    varying vec2 vUv;\n\n    void main() {\n      vUv = uv;\n      gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);\n    }",
  fragmentShader: "\n    #include <common>\n\n    uniform float strength;\n    uniform float minLuminance;\n    uniform float maxLuminance;\n    uniform sampler2D tDiffuse;\n    uniform sampler2D tDirt;\n    varying vec2 vUv;\n\n    void main() {\n      vec4 colorDirt = texture2D(tDirt, vUv);\n      vec4 colorGame = texture2D(tDiffuse, vUv);\n\n      float luminance = linearToRelativeLuminance(colorGame.rgb);\n      float amount = smoothstep(minLuminance, maxLuminance, luminance) * strength;\n\n      gl_FragColor = colorGame + colorDirt * amount;\n    }"
};

var useEffectPass = function useEffectPass(pass) {
  var composer = useComposer();
  useEffect(function () {
    composer.addPass(pass);
    return function () {
      return composer.removePass(pass);
    };
  }, [composer, pass]);
};

var useShaderUniforms = function useShaderUniforms(pass, uniforms) {
  useEffect(function () {
    for (var uniform in uniforms) {
      pass.uniforms[uniform].value = uniforms[uniform];
    }
  }, [pass].concat(_toConsumableArray(Object.values(uniforms))));
};

var LensDirt = function LensDirt(_ref) {
  var texture = _ref.texture,
      _ref$strength = _ref.strength,
      strength = _ref$strength === void 0 ? 1 : _ref$strength;
  var pass = useMemo(function () {
    return new ShaderPass(LensDirtShader);
  }, []);
  var tDirt = useMemo(function () {
    return new TextureLoader().load(texture);
  }, [texture]);
  useShaderUniforms(pass, {
    tDirt: tDirt,
    strength: strength
  });
  useEffectPass(pass);
  return null;
};

var RenderPass = function RenderPass(_ref) {
  var scene = _ref.scene,
      camera = _ref.camera;
  var pass = useMemo(function () {
    return new RenderPass$1(scene, camera);
  }, [scene, camera]);
  useEffectPass(pass);
  return null;
};

var UnrealBloomPass = function UnrealBloomPass(_ref) {
  var _ref$size = _ref.size,
      size = _ref$size === void 0 ? [window.innerWidth, window.innerHeight] : _ref$size,
      _ref$strength = _ref.strength,
      strength = _ref$strength === void 0 ? 1 : _ref$strength,
      _ref$radius = _ref.radius,
      radius = _ref$radius === void 0 ? 1 : _ref$radius,
      _ref$threshold = _ref.threshold,
      threshold = _ref$threshold === void 0 ? 0.9 : _ref$threshold;
  var pass = useMemo(function () {
    return new UnrealBloomPass$1(_construct(Vector2, _toConsumableArray(size)), strength, radius, threshold);
  }, [].concat(_toConsumableArray(size), [strength, radius, threshold]));
  useEffectPass(pass);
  return null;
};

var Vignette = function Vignette(_ref) {
  var _ref$offset = _ref.offset,
      offset = _ref$offset === void 0 ? 0.5 : _ref$offset,
      _ref$darkness = _ref.darkness,
      darkness = _ref$darkness === void 0 ? 2 : _ref$darkness;
  var pass = useMemo(function () {
    return new ShaderPass(VignetteShader);
  }, []);
  useShaderUniforms(pass, {
    offset: offset,
    darkness: darkness
  });
  useEffectPass(pass);
  return null;
};

var _excluded$2 = ["children"];
var BasicRenderPipeline = function BasicRenderPipeline(_ref) {
  var scene = _ref.scene,
      camera = _ref.camera,
      children = _ref.children;
  return /*#__PURE__*/React.createElement(Composer, null, /*#__PURE__*/React.createElement(RenderPass, {
    scene: scene,
    camera: camera
  }), children);
};
var FancyRenderPipeline = function FancyRenderPipeline(_ref2) {
  var children = _ref2.children,
      props = _objectWithoutProperties(_ref2, _excluded$2);

  return /*#__PURE__*/React.createElement(BasicRenderPipeline, props, /*#__PURE__*/React.createElement(UnrealBloomPass, {
    strength: 1.5,
    radius: 0.8,
    threshold: 0.3
  }), /*#__PURE__*/React.createElement(EffectPass, {
    pass: AdaptiveToneMappingPass,
    args: [true, 256]
  }), /*#__PURE__*/React.createElement(Vignette, null), children);
};

function useNullableState(initial) {
  return useState(initial);
}

var ApplicationContext = /*#__PURE__*/createContext(null);
var useApplication = function useApplication() {
  return useContext(ApplicationContext);
};
var Application = function Application(_ref3) {
  var children = _ref3.children,
      _ref3$renderPipeline = _ref3.renderPipeline,
      RenderPipeline = _ref3$renderPipeline === void 0 ? BasicRenderPipeline : _ref3$renderPipeline;

  var _useNullableState = useNullableState(),
      _useNullableState2 = _slicedToArray(_useNullableState, 2),
      scene = _useNullableState2[0],
      setScene = _useNullableState2[1];

  var _useNullableState3 = useNullableState(),
      _useNullableState4 = _slicedToArray(_useNullableState3, 2),
      camera = _useNullableState4[0],
      setCamera = _useNullableState4[1];

  useWindowResizeHandler(function () {
    if (camera) {
      var width = window.innerWidth;
      var height = window.innerHeight;

      if (camera instanceof THREE.PerspectiveCamera) {
        camera.aspect = width / height;
        camera.updateProjectionMatrix();
      }
    }
  }, [camera]);
  return /*#__PURE__*/React.createElement(Ticker, null, /*#__PURE__*/React.createElement(Renderer, null, scene && camera && /*#__PURE__*/React.createElement(RenderPipeline, {
    scene: scene,
    camera: camera
  }), /*#__PURE__*/React.createElement(T$1.Scene, {
    ref: setScene
  }, /*#__PURE__*/React.createElement(ApplicationContext.Provider, {
    value: {
      setCamera: setCamera,
      setScene: setScene
    }
  }, children instanceof Function ? children({
    setScene: setScene,
    setCamera: setCamera
  }) : children))));
};

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var _excluded$1 = ["instanceLimit"];
/* Create a local reactor with the Three.js classes we need */

var T = makeReactor({
  Group: Group,
  InstancedMesh: InstancedMesh,
  Object3D: Object3D
});
var makeInstanceComponents = function makeInstanceComponents(_ref) {
  var systemFactory = _ref.systemFactory,
      entityFactory = _ref.entityFactory,
      _ref$usage = _ref.usage,
      usage = _ref$usage === void 0 ? DynamicDrawUsage : _ref$usage;

  /* We're using Miniplex as a state container. */
  var ECS = createECS();
  /* If a system factory has been passed, prepare the custom system. */

  var system = systemFactory && systemFactory(ECS.world);
  /* This component renders the InstancedMesh itself and continuously updates it
     from the data in the ECS. */

  var Root = function Root(_ref2) {
    var _ref2$instanceLimit = _ref2.instanceLimit,
        instanceLimit = _ref2$instanceLimit === void 0 ? 10000 : _ref2$instanceLimit,
        props = _objectWithoutProperties(_ref2, _excluded$1);

    var instancedMesh = useRef(null);
    useEffect(function () {
      instancedMesh.current.instanceMatrix.setUsage(usage);
    }, []);

    var _ECS$useArchetype = ECS.useArchetype("transform", "visible"),
        entities = _ECS$useArchetype.entities;

    function updateInstanceMatrix() {
      var imesh = instancedMesh.current;
      var l = entities.length;
      var count = 0;

      for (var i = 0; i < l; i++) {
        var _entities$i = entities[i],
            transform = _entities$i.transform,
            visible = _entities$i.visible;

        if (visible) {
          imesh.setMatrixAt(i, transform.matrixWorld);
          count++;
        }
      }

      imesh.instanceMatrix.needsUpdate = true;
      imesh.count = count;
    }

    useTicker("render", function (dt) {
      system === null || system === void 0 ? void 0 : system(dt);
      updateInstanceMatrix();
    });
    return /*#__PURE__*/React.createElement(T.InstancedMesh, _extends({}, props, {
      ref: instancedMesh,
      args: [null, null, instanceLimit]
    }));
  };

  var useInstances = function useInstances() {
    var count = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
    return ECS.useEntities(count, function () {
      return _objectSpread2(_objectSpread2({}, entityFactory ? entityFactory() : {}), {}, {
        instance: Tag,
        transform: new Object3D(),
        visible: true
      });
    });
  };

  var Instance = /*#__PURE__*/forwardRef(function (props, ref) {
    return /*#__PURE__*/React.createElement(T.Object3D, _extends({}, props, {
      object: useInstances(1)[0].transform,
      ref: ref
    }));
  });

  var ThinInstance = function ThinInstance(_ref3) {
    var _ref3$count = _ref3.count,
        count = _ref3$count === void 0 ? 1 : _ref3$count;
    useInstances(count);
    return null;
  };

  return {
    world: ECS.world,
    useArchetype: ECS.useArchetype,
    useInstances: useInstances,
    Root: Root,
    Instance: Instance,
    ThinInstance: ThinInstance
  };
};

var loadedResources = {};
var useLoader = function useLoader(Loader, url) {
  /* If there's already a cached entry, return or throw it */
  if (loadedResources[url]) {
    if (loadedResources[url] instanceof Promise) {
      throw loadedResources[url];
    } else {
      return loadedResources[url];
    }
  }
  /* If we got here, it means that the resource has not been loaded, and there have been
  no previous attempts at loading it, so let's get to it! */

  /* Create a promise */


  var promise = new Promise(function (resolve) {
    new Loader().load(url, resolve);
  });
  /* Cache it, so subsequent invocations of this method don't repeat it */

  loadedResources[url] = promise;
  /* When the promise succeeds, replace the cached entry with its return value */

  promise.then(function (data) {
    loadedResources[url] = data;
  });
  /* Finally, throw the promise to trigger Suspense */

  throw promise;
};

var _excluded = ["children"];
var Text = function Text(_ref) {
  var children = _ref.children,
      props = _objectWithoutProperties(_ref, _excluded);

  var troika = useConst(function () {
    return new Text$1();
  });
  /* Separate plaintext nodes from the rest */

  var _React$useMemo = React.useMemo(function () {
    var nodes = [];
    var text = "";
    React.Children.forEach(children, function (child) {
      if (typeof child === "string" || typeof child === "number") {
        text += child;
      } else {
        nodes.push(child);
      }
    });
    return [nodes, text];
  }, [children]),
      _React$useMemo2 = _slicedToArray(_React$useMemo, 2),
      nodes = _React$useMemo2[0],
      text = _React$useMemo2[1];
  /* Sync text object every time props change */


  useLayoutEffect(function () {
    return troika.sync();
  });
  return /*#__PURE__*/React.createElement(Primitive, _extends({
    object: troika,
    text: text
  }, props), nodes);
};

/* Components */

export { Application, BasicRenderPipeline, Composer, EffectPass, EventHandling, FancyRenderPipeline, LensDirt, On, OnWindowResize, RenderPass, Renderer, Text, Ticker, UnrealBloomPass, Update, Vignette, T$1 as default, makeInstanceComponents, useAnimationFrame, useApplication, useComposer, useEffectPass, useEventListener, useLoader, useRenderer, useShaderUniforms, useTicker, useWindowResizeHandler };
